MODULE obc_par
   !!==============================================================================
   !!                  ***  MODULE obc_par   ***
   !! Open Boundary Cond. :   define related parameters
   !!==============================================================================
#if defined key_obc
   !!----------------------------------------------------------------------
   !!   'key_obc' :                                 Open Boundary Condition
   !!----------------------------------------------------------------------
   !! history :
   !!  8.0   01/91   (CLIPPER)  Original code 
   !!  9.0   06/02   (C. Talandier)  modules
   !!----------------------------------------------------------------------
   !! * Modules used
   USE par_oce         ! ocean parameters

   IMPLICIT NONE
   PUBLIC

   LOGICAL, PUBLIC, PARAMETER ::   lk_obc = .TRUE.   !: Ocean Boundary Condition flag

# if defined key_eel_r5
   !!----------------------------------------------------------------------
   !!   'key_eel_r5' :                                 EEL R5 configuration
   !!----------------------------------------------------------------------
#    include "obc_par_EEL_R5.h90"

# else
   !!---------------------------------------------------------------------
   !! open boundary parameter
   !!---------------------------------------------------------------------
   !! * EAST open boundary
   LOGICAL, PARAMETER ::     &  !:
      lp_obc_east = .FALSE.     !: to active or not the East open boundary
   INTEGER, PARAMETER ::     &  !:
      jpieob  = jpiglo-2,    &  !: i-localization of the East open boundary (must be ocean U-point)
      jpjed   =        2,    &  !: j-starting indice of the East open boundary (must be land T-point)
      jpjef   = jpjglo-1,    &  !: j-ending   indice of the East open boundary (must be land T-point)
      jpjedp1 =  jpjed+1,    &  !: first ocean point         "                 "
      jpjefm1 =  jpjef-1        !: last  ocean point         "                 "

   !! * WEST open boundary
   LOGICAL, PARAMETER ::     &  !:
      lp_obc_west = .FALSE.     !: to active or not the West open boundary
   INTEGER, PARAMETER ::     &  !:
      jpiwob  =	       2,    &  !: i-localization of the West open boundary (must be ocean U-point)
      jpjwd   =	       2,    &  !: j-starting indice of the West open boundary (must be land T-point)
      jpjwf   = jpjglo-1,    &  !: j-ending   indice of the West open boundary (must be land T-point)
      jpjwdp1 =  jpjwd+1,    &  !: first ocean point         "                 "
      jpjwfm1 =  jpjwf-1        !: last  ocean point         "                 "

   !! * NORTH open boundary
   LOGICAL, PARAMETER ::     &  !:
      lp_obc_north = .FALSE.    !: to active or not the North open boundary
   INTEGER, PARAMETER ::     &  !:
      jpjnob  = jpjglo-2,    &  !: j-localization of the North open boundary (must be ocean V-point)
      jpind   =        2,    &  !: i-starting indice of the North open boundary (must be land T-point)
      jpinf   = jpiglo-1,    &  !: i-ending   indice of the North open boundary (must be land T-point)
      jpindp1 =  jpind+1,    &  !: first ocean point         "                 "
      jpinfm1 =  jpinf-1        !: last  ocean point         "                 "

   !! * SOUTH open boundary
   LOGICAL, PARAMETER ::     &  !:
      lp_obc_south = .FALSE.    !: to active or not the South open boundary
   INTEGER, PARAMETER ::     &  !:
      jpjsob  =        2,    &  !: j-localization of the South open boundary (must be ocean V-point)
      jpisd   =        2,    &  !: i-starting indice of the South open boundary (must be land T-point)
      jpisf   = jpiglo-1,    &  !: i-ending   indice of the South open boundary (must be land T-point)
      jpisdp1 =  jpisd+1,    &  !: first ocean point         "                 "
      jpisfm1 =  jpisf-1        !: last  ocean point         "                 "
   
   INTEGER, PARAMETER ::     &  !:
      jpnic = 2700              !: maximum number of isolated coastlines points 

# endif

#else
   !!----------------------------------------------------------------------
   !!   Default option :                         NO open boundary condition
   !!----------------------------------------------------------------------
   LOGICAL, PUBLIC, PARAMETER ::   lk_obc = .FALSE.  !: Ocean Boundary Condition flag
#endif

   !!======================================================================
END MODULE obc_par
