C $Id$
CDIR$ LIST
      SUBROUTINE trcage(kt)
CCC---------------------------------------------------------------------
CCC
CCC                       ROUTINE trcage
CCC                     *******************
CCC
CCC  PURPOSE :
CCC  ---------
CCC     Compute the source for the age tracer
CCC
CCC
CC   METHOD :
CC   -------
CC
CC      multitasked on vertical slab (jj-loop)
CC      surface concentration is restored to 0 with a 
CC      time constant set to 7 days
CC
CC
CC   INPUT :
CC   -----
CC      argument
CC              ktask           : task identificator
CC              kt              : time step
CC      COMMON
CC            /comcoo/          : orthogonal curvilinear coordinates
CC                                and scale factors
CC                                depths
CC            /comtsk/          : multitasking
CC
CC   OUTPUT :
CC   ------
CC
CC   WORKSPACE :
CC   ---------
CC      COMMON
CC
CC   EXTERNAL :                   no
CC   --------
CC
CC   REFERENCES :                 no
CC   ----------
CC
CC   MODIFICATIONS:
CC   --------------
CC       original : 02-05 (K. Rodgers and O. Aumont)
CC----------------------------------------------------------------------

      USE oce_trc
      USE trp_trc
      USE sms
      IMPLICIT NONE

CCC---------------------------------------------------------------------
CCC  OPA8, LODYC (11/96)
CCC---------------------------------------------------------------------
CC----------------------------------------------------------------------
CC local declarations
CC ==================
      INTEGER kt

#if defined key_passivetrc
#    if defined key_trc_age
C
      INTEGER ji,jj,jk
CC----------------------------------------------------------------------

CCC---------------------------------------------------------------------
CCC  OPA8, LODYC (15/11/96)
CCC---------------------------------------------------------------------
C
C
C 1. Restore surface concentrations to 0 for age concentration
C    and 1 for component concentration
C ------------------------------------------------------------
C
       DO jj = 1,jpj
         DO ji = 1,jpi
             tra(ji,jj,1,2)=-trn(ji,jj,1,2)/(tauage1*rjjss)
             tra(ji,jj,1,1)=(1.-trn(ji,jj,1,1))/(tauage1*rjjss)
         END DO
       END DO
C
C 2. Increase sub-surface age concentrations so that the 
C    concentration is increased by 1 after tauage years
C --------------------------------------------------
C
        DO jk = 2,jpk
          DO jj = 1,jpj
            DO ji = 1,jpi
               tra(ji,jj,jk,2)=tra(ji,jj,jk,2)+trn(ji,jj,jk,1)
     &                          /(tauage2*raass)
            END DO
          END DO
        END DO
C
#    else
C
C    No optical model
C
#    endif
#else
C
C no passive tracers
C
#endif
C
      RETURN
      END
