MODULE trcstp
   !!======================================================================
   !!                       ***  MODULE trcstp  ***
   !! Time-stepping    : time loop of opa for passive tracer
   !!======================================================================
#if defined key_passivetrc
   !!----------------------------------------------------------------------
   !!   trc_stp      : passive tracer system time-stepping
   !!----------------------------------------------------------------------
   !! * Modules used
   USE oce_trc          ! ocean dynamics and active tracers variables
   USE trc              ! ocean passive tracers variables 
   USE trctrp           ! passive tracers transport

   IMPLICIT NONE
   PRIVATE

   !! * Routine accessibility
   PUBLIC trc_stp           ! called by step
   !!----------------------------------------------------------------------
   !!   OPA 9.0 , LODYC-IPSL (2003)
   !!----------------------------------------------------------------------

CONTAINS

   SUBROUTINE trc_stp( kt )
      !!-------------------------------------------------------------------
      !!                     ***  ROUTINE trc_stp  ***
      !!                      
      !! ** Purpose : Time loop of opa for passive tracer
      !! 
      !! ** Method  : 
      !!              Compute the passive tracers trends 
      !!              Update the passive tracers
      !!
      !! History :
      !!   9.0  !  04-03  (C. Ethe)  Original
      !!-------------------------------------------------------------------
      !! * Arguments
      INTEGER, INTENT( in ) ::  kt  ! ocean time-step index


      ! this ROUTINE is called only every ndttrc time step
      IF( MOD( kt , ndttrc ) /= 0 ) RETURN

      ! tracers: sink and source 


      CALL trcsms( kt )
 

      ! transport of passive tracers

      CALL trc_trp( kt )


   END SUBROUTINE trc_stp

#else
   !!----------------------------------------------------------------------
   !!   Default key                                     NO passive tracers
   !!----------------------------------------------------------------------
CONTAINS
   SUBROUTINE trc_stp( kt )        ! Empty routine
      INTEGER, INTENT(in) :: kt
      WRITE(*,*) 'trc_stp: You should not have seen this print! error?', kt
   END SUBROUTINE trc_stp
#endif

   !!======================================================================
END MODULE trcstp
