      SUBROUTINE trcctl
!!
!!
!!                       ROUTINE trcctl
!!                     ******************
!!
!!  PURPOSE :
!!  ---------
!!     only for passive tracer
!!     control the cpp options for the run and IF files are availables
!!     control also consistancy between options and namelist values
!!
!!   METHOD :
!!   -------
!!      we use IF/ENDIF inside #IF defined option-cpp
!!      c a u t i o n : FILE name must not exceed 21 characters
!!      -------------
!!
!!   INPUT :                      no
!!   -----
!!
!!   OUTPUT :                     no
!!   ------
!!
!!   WORKSPACE :
!!   ---------
!!      local
!!           clold,clnew,clfor,clunf,clseq,cldir,clname,
!!           ildta,ilglo,ibloc,ilseq
!!
!!   EXTERNAL :
!!   --------
!!
!!   MODIFICATIONS:
!!   --------------
!!      original :
!!                 04/00 (O. Aumont, M.A. Foujols) HAMOCC3 and P3ZD
!!      additions : 00/05 (A. Estublier) TVD Limiter Scheme
!!      additions : 00/06 (A. Estublier) MUSCL Scheme
!!      additions : 00/11 (MA Foujols, E Kestenare) Lateral diffusion option
!!      additions : 00/12  ( E Kestenare): improve controls of defined keys
!! 
!!----------------------------------------------------------------------
!! parameters and commons
!! ======================
!!
      USE oce_trc
      USE trc
      USE sms
      USE trctrp_ctl
      IMPLICIT NONE
!!
!!----------------------------------------------------------------------
!! local declarations
!! ==================

#if defined key_passivetrc
      CHARACTER*32 clname,clold,clfor,clseq,clnew,cldir,clunf,clunk
      INTEGER iused(1,100),ilu
      INTEGER ildta,ilglo,ibloc,ilseq,istop
      INTEGER jn
!!!---------------------------------------------------------------------
!!!  OPA8, LODY! (15/11/96)
!!!---------------------------------------------------------------------

! 1. initialization
! -----------------

! 0. Parameter

      istop = 0

! 1. OPEN specifier

      clold='OLD'
      clnew='NEW'
      clunk='UNKNOWN'
      clfor='FORMATTED'
      clunf='UNFORMATTED'
      clseq='SEQUENTIAL'
      cldir='DIRECT'

! 2. SEQUENTIAL value

      ilseq=1

      ilu=0

! computation of the record length for direct access FILE
! this length depend of 512 for the t3d machine

      ibloc=512
      ildta=ibloc*((jpidta*jpjdta)/ibloc+1)*jpbyt
      ilglo=ibloc*((jpiglo*jpjglo)/ibloc+1)*jpbyt

! 3. LOGICAL UNIT initialization for specifi! files for passive tracer

!     nutwrs : OUTPUT for passive tracer restart UNIT (always used)
!     nutrst : restart FILE  INPUT  UNIT (lrsttr=.TRUE.)
!     nutini(jptra) : UNIT for initial FILE for tracer

      nutwrs = 72
      nutrst = 73

! 4. FILE for restart (output)

#if defined key_mpp || defined key_fdir
      clname='trc.restart.output'
      CALL ctlopn(nutwrs,clname,clunk,clunf,cldir,
     $    ilglo,ilu,iused,numout,lwp,1)
#endif

! 5. restart for passive tracer (input)
! -----------------------------

      IF(lwp) THEN
          WRITE(numout,*) ' '
          WRITE(numout,*) ' *** PASSIVE TRACER MODEL OPTIONS'
          WRITE(numout,*) ' *** CONTROL'
          WRITE(numout,*) ' '
      ENDIF

      IF(lwp) THEN
          WRITE(numout,*) ' '
          WRITE(numout,*) ' *** restart option for passive tracer'
          WRITE(numout,*) ' '
      ENDIF

      IF(lrsttr) THEN
          IF(lwp) THEN
              WRITE(numout,*) ' READ a restart FILE for passive tracer'
              WRITE(numout,*) ' '
          ENDIF

!         NetCDF FORMAT, see in the dtrlec routine
          trestart='initial.trc.nc'

#if defined key_mpp || defined key_fdir
          clname='trc.restart'
          CALL ctlopn(nutrst,clname,clunk,clunf,cldir,
     $        ilglo,ilu,iused,numout,lwp,1)
#endif

          IF(lwp) THEN
              IF(nrsttr.eq.0) THEN
                  WRITE(numout,*) ' nrsttr = 0 we dont control the date'
                  WRITE(numout,*) ' '
              ELSE IF(nrsttr.eq.1) THEN
                  WRITE(numout,*) ' nrsttr = 1 we control the date'
                  WRITE(numout,*) ' '
              ELSE
                  WRITE(numout,*) '  ===>>>> nrsttr is not egal 0 or 1'
                  WRITE(numout,*) ' =======                     ======'
                  WRITE(numout,*) ' we dont control the date'
                  WRITE(numout,*) ' '
              ENDIF
          ENDIF
      ELSE
          IF(lwp) THEN
              WRITE(numout,*) ' no restart FILE'
              WRITE(numout,*) ' '
              WRITE(numout,*) ' the PARAMETER nrsttr is not used'
              WRITE(numout,*) ' '
              IF(nrsttr.eq.1) THEN
                  WRITE(numout,*) ' nrsttr = 1 '
                  WRITE(numout,*) ' '
                  WRITE(numout,*) ' ===>>>> perhaps it is a mistake'
                  WRITE(numout,*) ' ======= '
                  WRITE(numout,*) ' '
              ENDIF
          ENDIF

! 6. OPEN FILES for initial tracer value

          DO jn=1,jptra

! OPEN input FILE only IF lutini(jn) is true

            IF (lutini(jn)) THEN  

! prepare input FILE name a
!                        
                IF (lwp) THEN 
                    WRITE(numout,*)
     $                  ' READ an initial FILE :',
     $                  ' for passive tracer number :',jn
     $                  ,' traceur : ',ctrcnm(jn) 
                    WRITE(numout,*) ' '
                END IF 
            END IF 
          END DO   
      ENDIF

! 7. Don't USE non penetrative convective mixing option
!     it's not implemented for passive tracer

#if defined key_convnpc
      IF (lwp) THEN
          WRITE (numout,*) ' ===>>>> : w a r n i n g '
          WRITE (numout,*) ' =======   ============= '
          WRITE (numout,*) ' STOP, this sheme is not implemented'
          WRITE (numout,*) ' in passive tracer model:'
          WRITE (numout,*) ' non penetrative convect. mixing scheme'
      ENDIF 
      istop = istop + 1
#endif


! 8. transport scheme option
! --------------------------

      WRITE(numout,*) '  '
      WRITE(numout,*) '  '
      CALL trc_trp_ctl
      WRITE(numout,*) '  '
      WRITE(numout,*) '  '

! 9. SMS model
! ---------------------------------------------
C
      IF(lwp) THEN
          WRITE(numout,*) '  '
          WRITE(numout,*) ' *** Source/Sink model option'
          WRITE(numout,*) '  '
      ENDIF

#    if defined key_trc_npzd && defined key_trc_lobster1
      IF (lwp) THEN 
          WRITE (numout,*) ' ===>>>> : w a r n i n g '
          WRITE (numout,*) ' =======   ============= '
          WRITE (numout,*)
     $         ' STOP, only one model can be specified '
      END IF 
      istop = istop + 1
#    endif
#    if defined key_trc_npzd && defined key_trc_hamocc3
      IF (lwp) THEN 
          WRITE (numout,*) ' ===>>>> : w a r n i n g '
          WRITE (numout,*) ' =======   ============= '
          WRITE (numout,*)
     $         ' STOP, only one model can be specified '
      END IF 
      istop = istop + 1
#    endif
#    if defined key_trc_pisces && defined key_trc_lobster1
      IF (lwp) THEN 
          WRITE (numout,*) ' ===>>>> : w a r n i n g '
          WRITE (numout,*) ' =======   ============= '
          WRITE (numout,*)
     $         ' STOP, only one model can be specified '
      END IF 
      istop = istop + 1
#    endif
#    if defined key_trc_pisces && defined key_trc_npzd
      IF (lwp) THEN
          WRITE (numout,*) ' ===>>>> : w a r n i n g '
          WRITE (numout,*) ' =======   ============= '
          WRITE (numout,*)
     $         ' STOP, only one model can be specified '
      END IF
      istop = istop + 1
#    endif
#    if defined key_trc_pisces && defined key_trc_hamocc3
      IF (lwp) THEN
          WRITE (numout,*) ' ===>>>> : w a r n i n g '
          WRITE (numout,*) ' =======   ============= '
          WRITE (numout,*)
     $         ' STOP, only one model can be specified '
      END IF
      istop = istop + 1
#    endif
#    if defined key_trc_npzd
#     include "trcctl.npzd.h" 
#    elif defined key_trc_lobster1
#     include "trcctl.lobster1.h"
#    elif defined key_trc_pisces
#     include "trcctl.pisces.h"
#    elif defined key_trc_hamocc3
#        if defined key_trc_p3zd
#     include "trcctl.p3zd.h"
#        else
#     include "trcctl.hamocc3.h"
#        endif
#    else
      IF (lwp) THEN 
          WRITE (numout,*) ' No Source/Sink model '
          WRITE (numout,*) ' '
      END IF 
#    endif

! E r r o r  control
! ------------------
      IF ( istop .GT. 0  ) THEN
          IF(lwp)WRITE(numout,*)
          IF(lwp)WRITE(numout,*) istop,' E R R O R found : we stop'
          IF(lwp)WRITE(numout,*) '**************************'
          IF(lwp)WRITE(numout,*)
          STOP 'trcctl'
      ENDIF

#else

! no passive tracers

#endif

      RETURN
      END SUBROUTINE trcctl
