CCC $Header$
CDIR$ LIST
      SUBROUTINE trclec
CCC---------------------------------------------------------------------
CCC
CCC                       ROUTINE trclec
CCC                     ******************
CCC
CCC  PURPOSE :
CCC  ---------
CCC     READ and PRINT options for the passive tracer run (namelist)
CCC
CC   METHOD :                   : no
CC   -------
CC
CC   INPUT :
CC   -----
CC      the namelist FILE ( UNIT numnat ) :
CC            &nattrc           : general
CC            &natnum           : numerical schemes
CC            &natrtd           : dynamical trends 
CC            &natadd           : additional arrays for diagnostics
CC
CC   OUTPUT :
CC   ------
CC      COMMON                  : (#ifdef "key_passivetrc")
CC            /cittrc/          : tracer names
CC            /cot3ad/          : options for smolarkiewicz scheme
CC            /cit3ad/          : other options for smolarkiewicz scheme
CC            /cottrd/          : passive tracer trends (#ifef
CC                                 "key_trc_diatrd")
CC
CC   WORKSPACE :                : no
CC   ---------
CC
CC   MODIFICATIONS:
CC   --------------
CC      original  : 96-11 (M.A. Foujols, M. Levy) passive tracer
CC      modification : 98-04 (M.A Foujols, L. Bopp) ahtrb0 for isopycnal
CC                                                  diffusion
CC      modification : 99-10(M.A. Foujols, M. Levy) separation of sms
CC      additions : 00-05(A. Estublier) TVD Limiter Scheme : Tests 
CC                                      on ndttrc
CC      additions : 00-06(A. Estublier) MUSCL Scheme : Tests 
CC                                      on ndttrc
CC      additions : 00-07(A. Estublier) PPM Scheme : Tests on ndttrc
CC      modification : 00-11 (M.A Foujols, E Kestenare) trcrat, ahtrc0 and aeivtr0
CC      modification : 01-01 (E Kestenare) suppress ndttrc=1 
CC                                         for Arakawa and TVD schemes
CC----------------------------------------------------------------------
CC parameters and commons
CC ======================
CDIR$ NOLIST
      USE oce_trc
      USE trc
      USE trctrp_lec
      IMPLICIT NONE
CDIR$ LIST
CC----------------------------------------------------------------------
CC local declarations
CC ==================

#if defined key_passivetrc
      INTEGER ji
      CHARACTER*32 clname
CC----------------------------------------------------------------------
CC statement functions
CC ===================
CDIR$ NOLIST
#include "passivetrc_substitute.h90"
CDIR$ LIST
CC
CCC---------------------------------------------------------------------
CCC  OPA8, LODYC (15/11/96)
CCC---------------------------------------------------------------------
C
C 0. initializations
C ------------------
C
      namelist/nattrc/nwritetrc,lrsttr,nrsttr, ctrcnm,ctrcnl,ctrcun !general
     $    ,lutini,ijulian

      namelist/natnum/rsc,rtrn,ncortrc,ndttrc,crosster

#    if defined key_trc_diatrd
      namelist/natrtd/luttrd,nwritetrd                      ! dynamical trends
#    endif

#    if defined key_trc_diaadd
      namelist/natadd/ctrc3d,ctrc3l,ctrc2d,ctrc2l, ctrc3u, ctrc2u,
     $    nwriteadd             !additional diagnostics
#    endif

C
C

      IF(lwp) THEN
          WRITE(numout,*) ' '
          WRITE(numout,*) ' ROUTINE trclec'
          WRITE(numout,*) ' **************'
          WRITE(numout,*) ' '
          WRITE(numout,*) ' namelist for passive tracer'
          WRITE(numout,*) ' ***************************'
          WRITE(numout,*) ' '
      ENDIF
C
      numnat=80
      REWIND (numnat)
C
      clname='namelist.passivetrc'
      OPEN( numnat, FILE= clname, FORM='formatted', STATUS = 'old')

C
C
C 1., 2. & 3. initialization with namelist files
C ----------------------------------------------
C
C 1.0 namelist nattrc :
C
      nwritetrc = 10
      lrsttr=.FALSE.
      nrsttr = 0

      DO ji=1,jptra
        WRITE (ctrcnm(ji),'("TR_",I1)') ji
        WRITE (ctrcnl(ji),'("TRACER NUMBER ",I1)') ji
        ctrcun(ji)='mmole/m3'
        lutini(ji)=.FALSE. 
      END DO
C

      REWIND(numnat)
      READ(numnat,nattrc)
C
      IF(lwp) THEN
          WRITE(numout,*) ' '
          WRITE(numout,*) 'nattrc'
          WRITE(numout,*) ' '
          WRITE(numout,*)
     $        ' frequency of outputs for passive tracers nwritetrc = '
     $        ,nwritetrc  
          WRITE(numout,*) ' restart LOGICAL for passive tr. lrsttr = ',
     $        lrsttr
          WRITE(numout,*) ' control of time step for p. tr. nrsttr = ',
     $        nrsttr
          DO ji=1,jptra
            WRITE(numout,*) ' tracer nb: ',ji,' name = ',ctrcnm(ji)
     $          ,ctrcnl(ji) 
            WRITE(numout,*) ' in unit = ',ctrcun(ji)
            WRITE(numout,*) ' initial value in FILE : ',lutini(ji) 
            WRITE(numout,*) ' '
          END DO
          WRITE(numout,*) ' '
      ENDIF

# if defined key_trc_diatrd
C
C 1.2 namelist nattrd : passive tracers dynamical trends
C
      nwritetrd=10
C
C default : no dynamical trend recording
C
      DO ji=1,jptra
        luttrd(ji) = .FALSE.
      END DO 

      REWIND(numnat)
      READ(numnat,natrtd)
C
      IF(lwp) THEN
          WRITE(numout,*) 'natrtd'
          WRITE(numout,*) ' '
          WRITE(numout,*)
     $        ' frequency of outputs for dynamical trends nwritetrd = '
     $        ,nwritetrd
          DO ji=1,jptra
            WRITE(numout,*)
     $          ' keep dynamical trends for tracer number :',ji
     $          ,luttrd(ji)   
          END DO 
      ENDIF
# endif
C
C 1.3 namelist natadd : passive tracers diagnostics
C
# if defined key_trc_diaadd
C
      nwriteadd = 10
C
C default value for 3D output arrays : short and long name, units
C
      DO ji=1,jpdia3d
        WRITE (ctrc3d(ji),'("3D_",I1)') ji
        WRITE (ctrc3l(ji),'("3D DIAGNOSTIC NUMBER ",I2)') ji
        ctrc3u(ji)=' '
      END DO 
C
C
C default value for 2D output arrays : short and long name, units
C
      DO ji=1,jpdia2d
        WRITE (ctrc2d(ji),'("2D_",I1)') ji
        WRITE (ctrc2l(ji),'("2D DIAGNOSTIC NUMBER ",I2)') ji
        ctrc2u(ji)=' '
      END DO 
C
      REWIND(numnat)
      READ(numnat,natadd)
C
      IF(lwp) THEN
          WRITE(numout,*) ' natadd'
          WRITE(numout,*) ' '
          WRITE(numout,*)
     $        ' frequency of outputs for additional arrays nwriteadd = '
     $        ,nwriteadd
          DO ji=1,jpdia3d
            WRITE(numout,*)
     $          'name of 3d output field number :',ji,' : ',ctrc3d(ji)  
            WRITE(numout,*) ctrc3l(ji)  
            WRITE(numout,*) ' in unit = ',ctrc3u(ji)
          END DO 
          WRITE(numout,*) ' '
          DO ji=1,jpdia2d
            WRITE(numout,*)
     $          'name of 2d output field number :',ji,' : ',ctrc2d(ji)  
            WRITE(numout,*) ctrc2l(ji)  
            WRITE(numout,*) ' in unit = ',ctrc2u(ji)
          END DO 
          WRITE(numout,*) ' '
      ENDIF
#endif

C 1.1 namelist natnum :
C
      rsc=1.
      rtrn=1.e-15
      ncortrc=1
      ndttrc=4
      crosster=.FALSE.
C
      REWIND(numnat)
      READ(numnat,natnum)

Chris  computes the first time step of tracer model
      nittrc000 = nit000 + ndttrc - 1
Chris

C
      IF(lwp) THEN
          WRITE(numout,*) ' '
          WRITE(numout,*) 'natnum'
          WRITE(numout,*) ' '
          WRITE(numout,*) ' tuning coefficient              rsc     = ',
     $        rsc
          WRITE(numout,*) ' truncation value                rtrn    = ',
     $        rtrn
          WRITE(numout,*) ' number of corrective phase      ncortrc = ',
     $        ncortrc
          WRITE(numout,*) ' time step freq. for pass. trac. ndttrc  = ',
     $        ndttrc
          WRITE(numout,*) ' 1st time step for pass. trac. nittrc000 = ',
     $        nittrc000
          WRITE(numout,*) ' computes or not crossterms    crosster  = ',
     $        crosster
      ENDIF


C
C namelist of transport
C
      CALL trc_trp_lec
C
C namelist of SMS
      CALL trclsm

#else
C
C no passive tracers
C
#endif
      RETURN
      END
